local width = love.graphics.getWidth()
local height = love.graphics.getHeight() 
local dots = {}
local timer = 0
local RADIUS = 25

local function distanceBetweenPoints(x1, y1, x2, y2)
  local xdiff = x2 - x1
  local ydiff = y2 - y1
  local distance = math.sqrt(xdiff*xdiff + ydiff*ydiff)
  return distance
end

function love.load()
  love.graphics.setBackgroundColor(75, 75, 75)

  for i = 0, height / (RADIUS * 2) do
    for j = 0, width / (RADIUS * 2) do
      dot = {
        x = j * (RADIUS * 2),
        y = i * (RADIUS * 2),
        r = RADIUS,
        color = {255, 255, 255}
      }
      table.insert(dots, dot)
    end
  end
end

function love.update(dt)
  local mX = love.mouse.getX()
  local mY = love.mouse.getY()
  local centerX, centerY = width/2, height/2
  for i = 1, #dots do
    local d = dots[i]
    local normDiffX = (mX - centerX) / distanceBetweenPoints(mX, mY, centerX, centerY)
    local normDiffY = (mY - centerY) / distanceBetweenPoints(mX, mY, centerX, centerY)

    -- move
    d.x = d.x - normDiffX*200 * dt
    d.y = d.y - normDiffY*200 * dt
    
    -- wrap-around
    if d.x > width + d.r then  -- right edge 
      d.x = 0 - d.r
    end
    if d.y > height + d.r then  -- bottom edge 
      d.y = 0 - d.r
    end
    if d.x < 0 - d.r then  -- left edge
      d.x = width + d.r
    end
    if d.y < 0 - d.r then  -- top edge
      d.y = height + d.r
    end

    -- avoid mouse
    if distanceBetweenPoints(mX, mY, d.x, d.y) < d.r*2 then
      d.color = {
        love.math.random(100, 200),
        love.math.random(100, 200),
        love.math.random(100, 200)
      }

      d.r = d.r - 20 * dt

      if mX < d.x then
        d.x = d.x + 100 * dt
      elseif mX > d.x then
        d.x = d.x - 100 * dt
      end

      if mY < d.y then
        d.y = d.y + 100 * dt
      elseif mY > d.y then
        d.y = d.y - 100 * dt
      end
    end
  end
end

function love.draw()
  for i = 1, #dots do
    love.graphics.setColor(dots[i].color)
    love.graphics.circle('fill', dots[i].x, dots[i].y, dots[i].r)
    love.graphics.setColor(255, 255, 255)
  end
end

function love.keypressed(key)
  if key == 'escape' then
    love.event.quit()
  end

  if key == 'r' then
    dots = {}
    love.load()
  end
end